/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2013-2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  30.06.2018  mifi  First Version.
**************************************************************************/
#define __TALMEM_C__

/*
 * How to Allocate Dynamic Memory Safely
 * See: https://barrgroup.com/Embedded-Systems/How-To/Malloc-Free-Dynamic-Memory-Allocation
 *
 * Here the free list "First Fit" allocate and "Address Order" free management is used.
 */

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <string.h>
#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

typedef struct _mem_hdr_
{
   struct _mem_hdr_ *pNext;
   uint32_t          dSize;
} mem_hdr_t;   

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static mem_hdr_t *pFreelist = NULL;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  tal_MEMAdd                                                           */
/*                                                                       */
/*  Add the memroy to the dynamic memory system.                         */
/*                                                                       */
/*  In    : pBuffer, dSize                                               */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_MEMAdd (void *pBuffer, uint32_t dSize)
{
   uint32_t   dAddress;
   uint32_t   dRest;
   mem_hdr_t *pMem;   
   
   /* Align address to 8 */
   dAddress  = (uint32_t)pBuffer;
   dRest     = dAddress % 8;
   
   if ((dRest != 0) && (dSize > 16))
   {
      dRest     = 8 - dRest;
      dAddress += dRest;
      dSize    -= dRest;
   }
   
   if (dSize > 16)
   {
      /* Align size to 8 */
      dRest  = dSize % 8;
      dSize -= dRest;
   
      pMem = (mem_hdr_t*)dAddress;
      pMem->pNext = NULL;
      pMem->dSize = dSize;
      pMem = (mem_hdr_t*)((uint32_t)pMem + sizeof(mem_hdr_t));
   
      tal_MEMFree(pMem);
   }      

} /* tal_MEMAdd */

/*************************************************************************/
/*  tal_MEMMalloc                                                        */
/*                                                                       */
/*  Allocate memory of the given size.                                   */
/*  Returns NULL if memory cannot be allocated.                          */
/*                                                                       */
/*  In    : dSize                                                        */
/*  Out   : none                                                         */
/*  Return: p / NULL                                                     */
/*************************************************************************/
void *tal_MEMMalloc (uint32_t dSize)
{
   void      *p = NULL;
   mem_hdr_t *pPrev;
   mem_hdr_t *pMem;
   mem_hdr_t *pTmp;
   
   if (pFreelist != NULL)
   {
      /* Align to 8 */
      dSize = (dSize + 7) & ~7;
      
      /* Added header info */
      dSize += sizeof(mem_hdr_t);
   
      /* Check first element */
      if (pFreelist->dSize >= dSize)
      {
         pMem = pFreelist;
         
         if (pFreelist->dSize > dSize)
         {
            pFreelist = (mem_hdr_t*)((uint32_t)pFreelist + dSize);
            pFreelist->pNext = pMem->pNext;
            pFreelist->dSize = pMem->dSize - dSize;
         }
         else
         {
            pFreelist = pFreelist->pNext;
         }   
      }
      else
      {
         /* Check free list chain */
      
         pPrev = pFreelist;
         pMem  = pFreelist->pNext;
         
         while (pMem != NULL)
         {
            if (pMem->dSize == dSize)
            {
               pPrev->pNext = pMem->pNext;
               break;
            }
            
            if (pMem->dSize > dSize)
            {
               pTmp = (mem_hdr_t*)((uint32_t)pMem + dSize);
               pTmp->pNext = pMem->pNext;
               pTmp->dSize = pMem->dSize - dSize;
               
               pPrev->pNext = pTmp;
               break;
            }
         
            pPrev = pMem;
            pMem  = pMem->pNext;
         }
      }
      
      
      if (pMem != NULL)
      {
         pMem->pNext = NULL;
         pMem->dSize = dSize;
         p = (void*)((uint32_t)pMem + sizeof(mem_hdr_t));
      }
   }
   
   return(p);
} /* tal_MEMMalloc */

/*************************************************************************/
/*  tal_MEMCalloc                                                        */
/*                                                                       */
/*  Allocate memory of the given size, and initialize them to zero.      */
/*  Returns NULL if memory cannot be allocated.                          */
/*                                                                       */
/*  In    : dSize                                                        */
/*  Out   : none                                                         */
/*  Return: p / NULL                                                     */
/*************************************************************************/
void *tal_MEMCalloc (uint32_t dSize)
{
   void *p;
   
   p = tal_MEMMalloc(dSize);
   if (p != NULL)
   {
      memset(p, 0, dSize);
   }
   
   return(p);
} /* tal_MEMCalloc */

/*************************************************************************/
/*  tal_MEMFree                                                          */
/*                                                                       */
/*  Frees the allocated memory.                                          */
/*                                                                       */
/*  In    : pBuffer                                                      */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_MEMFree (void *pBuffer)
{
   mem_hdr_t *pPrev;
   mem_hdr_t *pNext;
   mem_hdr_t *pMem;

   if (pBuffer != NULL)
   {
      pMem = (mem_hdr_t*)((uint32_t)pBuffer - sizeof(mem_hdr_t));

      /* Check empty free list */
      if (NULL == pFreelist)
      {
         pFreelist = pMem;
         pFreelist->pNext = NULL;
      }
      /* Check in front of the free list, without space, mem + list */
      else if (((uint32_t)pMem + pMem->dSize) == (uint32_t)pFreelist)
      {
         pMem->pNext = pFreelist->pNext;
         pMem->dSize = pFreelist->dSize + pMem->dSize;
         pFreelist = pMem; 
      }
      /* Check in front of the free list, with space, mem |space| list */
      else if (((uint32_t)pMem + pMem->dSize) < (uint32_t)pFreelist)
      {
         pMem->pNext = pFreelist;
         pFreelist   = pMem;
      }
      else
      {
         /* Find correct position */
         pPrev = pFreelist;
         pNext = pFreelist->pNext;
         
         while ((pNext != NULL) && ((uint32_t)pNext < (uint32_t)pMem))
         {
            pPrev = pNext;
            pNext = pNext->pNext;
         }
         
         if (pNext != NULL)
         {
            /* Added in between pPrev and pNext */

            /* Check for prev + mem + next */
            if( (((uint32_t)pPrev + pPrev->dSize) == (uint32_t)pMem)  &&
                (((uint32_t)pMem  + pMem->dSize)  == (uint32_t)pNext) )
            {
               pPrev->pNext = pNext->pNext;
               pPrev->dSize = pPrev->dSize + pMem->dSize + pNext->dSize;
            }
            /* Check for prev + mem |space| next */                   
            else if (((uint32_t)pPrev + pPrev->dSize) == (uint32_t)pMem)
            {
               pPrev->pNext = pNext;
               pPrev->dSize = pPrev->dSize + pMem->dSize;
            }
            /* Check for prev |space| mem + next */                   
            else if (((uint32_t)pMem + pMem->dSize) == (uint32_t)pNext)
            {
               pPrev->pNext = pMem;
               pMem->pNext  = pNext->pNext;
               pMem->dSize  = pMem->dSize + pNext->dSize;
            }
            else
            {
               /* prev |space| mem |space| next */
               pMem->pNext  = pPrev->pNext;
               pPrev->pNext = pMem;
            }
         }
         else
         {
            /* Added at the end of the free list, prev */
            
            
            /* Check for prev + mem */                   
            if (((uint32_t)pPrev + pPrev->dSize) == (uint32_t)pMem)
            {
               pPrev->pNext = NULL;
               pPrev->dSize = pPrev->dSize + pMem->dSize;
            }
            else
            {
               /* prev |space| mem */
               pPrev->pNext = pMem;
               pMem->pNext  = NULL;
            }
         }
      }
   } /* end if (pBuffer != NULL) */

} /* tal_MEMFree */

/*** EOF ***/
